@@ -19,10 +19,33 @@
 """Helpers for an Ubuntu application."""
 import logging
 import os
+import sys
+from gi.repository import Gtk, GObject
 
 from . catfishconfig import get_data_file
 from . Builder import Builder
 
+python_version = sys.version_info[:3]
+gobject_version = GObject.pygobject_version
+gtk_version = (Gtk.get_major_version(),
+               Gtk.get_minor_version(),
+               Gtk.get_micro_version())
+
+
+def check_python_version(major_version, minor_version, micro=0):
+    """Return true if running python >= requested version"""
+    return python_version >= (major_version, minor_version, micro)
+
+
+def check_gtk_version(major_version, minor_version, micro=0):
+    """Return true if running gtk >= requested version"""
+    return gtk_version >= (major_version, minor_version, micro)
+
+
+def check_gobject_version(major_version, minor_version, micro=0):
+    """Return true if running gobject >= requested version"""
+    return gobject_version >= (major_version, minor_version, micro)
+
 
 def get_builder(builder_file_name):
     """Return a fully-instantiated Gtk.Builder instance from specified ui file
